## ----include=FALSE, message=FALSE--------------------
Samples <- paste('indiv_', seq(1:12), sep='')
Treatment <- paste('trt_', rep(1:3,4),sep='')
Table_1 <- cbind(Samples,Treatment)
colnames(Table_1) <- c('Samples','Treatment')

trt1 <- c(rep(1,4),rep(0,8))
trt2 <- c(rep(0,4),rep(1,4),rep(0,4))
trt3 <- c(rep(0,8),rep(1,4))
Table_2 <- cbind(Samples,trt1,trt2,trt3)
colnames(Table_2) <- c('Samples','trt1','trt2','trt3')


## ----plsda-tab, echo=FALSE---------------------------
knitr::kable(list(Table_1[1:10, ], Table_2[1:10, ]),
  caption = 'Example of an outcome factor y (left) transformed into an indicator dummy matrix Y (right). The transformation is performed internally in our supervised methods. Note that the third column is actually redundant as it can be built from the first two. ', booktabs = TRUE
)


## ----plsda-diag, echo = FALSE, fig.cap ='(ref:plsda-diag)', out.width = '80%'----
knitr::include_graphics('Figures/Part3/PLS-DA.pdf')


## ----results = 'hold', message=FALSE-----------------
library(mixOmics)
data(srbct)
X <- srbct$gene
# Outcome y that will be internally coded as dummy:
Y <- srbct$class 
dim(X); length(Y)


## ----------------------------------------------------
summary(Y)


## ---- eval = FALSE-----------------------------------
## plsda.result <- plsda(X, Y)     # 1 Run the method
## plotIndiv(plsda.result)         # 2 Plot the samples
## plotVar(plsda.result)           # 3 Plot the variables


## ---- eval = FALSE-----------------------------------
## splsda.result <- splsda(X, Y, keepX = c(50,30)) # 1 Run the method
## plotIndiv(splsda.result)                        # 2 Plot the samples
## plotVar(splsda.result)                          # 3 Plot the variables
## selectVar(splsda.result, comp = 1)$name         # Selected variables on comp 1
## plotLoadings(splsda.result, method = 'mean', contrib = 'max')


## ----plsda-pca, fig.cap='(ref:plsda-pca)'------------
pca.srbct <- pca(X, ncomp = 3, scale = TRUE)

plotIndiv(pca.srbct, group = srbct$class, ind.names = FALSE,
          legend = TRUE, 
          title = 'SRBCT, PCA comp 1 - 2')


## ----plsda-perf, fig.cap='(ref:plsda-perf)'----------
plsda.srbct <- plsda(X,Y, ncomp = 10)
set.seed(30) # For reproducibility with this handbook, remove otherwise
perf.plsda.srbct <- perf(plsda.srbct, validation = 'Mfold', folds = 3, 
                  progressBar = FALSE,   # Set to TRUE to track progress
                  nrepeat = 10)          # We suggest nrepeat = 50

plot(perf.plsda.srbct, sd = TRUE, legend.position = 'horizontal')


## ---- eval = FALSE-----------------------------------
## perf.plsda.srbct


## ----------------------------------------------------
plsda.srbct.final <- plsda(X,Y, ncomp = 3)


## ----plsda-indiv12, echo = TRUE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
plotIndiv(plsda.srbct.final, ind.names = FALSE, legend=TRUE,
          comp=c(1,2), ellipse = TRUE, 
          title = 'PLS-DA on SRBCT comp 1-2',
          X.label = 'PLS-DA comp 1', Y.label = 'PLS-DA comp 2')


## ----plsda-indiv13, echo = TRUE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
plotIndiv(plsda.srbct.final, ind.names = FALSE, legend=TRUE,
          comp=c(2,3), ellipse = TRUE, 
          title = 'PLS-DA on SRBCT comp 2-3',
          X.label = 'PLS-DA comp 2', Y.label = 'PLS-DA comp 3')


## ----plsda-plotindiv, fig.cap='(ref:plsda-plotindiv)', echo=FALSE, eval=TRUE, fig.align='center', out.width = '50%', fig.height=4, fig.ncol = 2, fig.subcap=c('', '')----
knitr::include_graphics(c('Figures/Part3/plsda-indiv12-1.png', 'Figures/Part3/plsda-indiv13-1.png'))


## ----------------------------------------------------
set.seed(30) # For reproducibility with this handbook, remove otherwise
perf.plsda.srbct.final <- perf(plsda.srbct.final, validation = 'Mfold', folds = 3, 
                  progressBar = FALSE,  # Det to TRUE to track progress
                  nrepeat = 50) 


## ----------------------------------------------------
perf.plsda.srbct.final$error.rate$BER[, 'max.dist']


## ----------------------------------------------------
perf.plsda.srbct.final$error.rate.class$max.dist


## ----plsda-background-max, echo = TRUE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
background.max <- background.predict(plsda.srbct.final, comp.predicted=2,
                                dist = 'max.dist') 
plotIndiv(plsda.srbct.final, comp = 1:2, group = srbct$class,
          ind.names = FALSE, title = 'Maximum distance',
          legend = TRUE,  background = background.max)


## ----plsda-background-cent, echo = FALSE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
background.cent <- background.predict(plsda.srbct.final, comp.predicted=2,
                                dist = 'centroids.dist') 
plotIndiv(plsda.srbct.final, comp = 1:2, group = srbct$class,
          ind.names = FALSE, title = 'Centroids distance',
          legend = TRUE,  background = background.cent)


## ----plsda-background-mah, echo = FALSE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
background.mah <- background.predict(plsda.srbct.final, comp.predicted=2,
                                dist = 'mahalanobis.dist') 
plotIndiv(plsda.srbct.final, comp = 1:2, group = srbct$class,
          ind.names = FALSE, title = 'Mahalanobis distance',
          legend = TRUE,  background = background.mah)


## ----plsda-background, fig.cap='(ref:plsda-background)', echo=FALSE, eval=TRUE, fig.cap='(ref:plsda-background)', fig.align='center', out.width = '30%', fig.height=4, fig.ncol = 3, fig.subcap=c('', '', '')----
knitr::include_graphics(c('Figures/Part3/plsda-background-max-1.png', 'Figures/Part3/plsda-background-cent-1.png', 'Figures/Part3/plsda-background-mah-1.png'))


## ----------------------------------------------------
# Grid of possible keepX values that will be tested for each comp
list.keepX <- c(1:10,  seq(20, 100, 10))
list.keepX


## ----------------------------------------------------
# This chunk takes ~ 2 min to run
# Some convergence issues may arise but it is ok as this is run on CV folds
tune.splsda.srbct <- tune.splsda(X, Y, ncomp = 4, validation = 'Mfold', 
                                 folds = 5, dist = 'max.dist', 
                                 test.keepX = list.keepX, nrepeat = 10)


## ----------------------------------------------------
# Just a head of the classification error rate per keepX (in rows) and comp
head(tune.splsda.srbct$error.rate)


## ----splsda-tune, fig.cap='(ref:splsda-tune)'--------
# The optimal number of components according to our one-sided t-tests
tune.splsda.srbct$choice.ncomp$ncomp

# The optimal keepX parameter according to minimal error rate
tune.splsda.srbct$choice.keepX
# To show the error bars across the repeats:
plot(tune.splsda.srbct, sd = TRUE)


## ----------------------------------------------------
# Optimal number of components based on t-tests on the error rate
ncomp <- tune.splsda.srbct$choice.ncomp$ncomp 
ncomp
# Optimal number of variables to select
select.keepX <- tune.splsda.srbct$choice.keepX[1:ncomp]  
select.keepX

splsda.srbct <- splsda(X, Y, ncomp = 3, keepX = select.keepX) 


## ----------------------------------------------------
set.seed(34)  # For reproducibility with this handbook, remove otherwise
perf.splsda.srbct <- perf(splsda.srbct, folds = 5, validation = "Mfold", 
                  dist = "max.dist", progressBar = FALSE, nrepeat = 10)
# perf.splsda.srbct  # Lists the different outputs
perf.splsda.srbct$error.rate


## ----------------------------------------------------
perf.splsda.srbct$error.rate.class


## ----splsda-stability, fig.cap='(ref:splsda-stability)', results='hold', fig.show='hold'----
par(mfrow=c(1,2))
# For comp 1
stable.comp1 <- perf.splsda.srbct$features$stable$comp1
barplot(stable.comp1, xlab = 'variables selected across CV folds', 
        ylab = 'Stability frequency',
        main = 'Feature stability for comp = 1')

# For comp2
stable.comp2 <- perf.splsda.srbct$features$stable$comp2
barplot(stable.comp2, xlab = 'variables selected across CV folds', 
        ylab = 'Stability frequency',
        main = 'Feature stability for comp = 2')
par(mfrow=c(1,1))


## ----------------------------------------------------
# First extract the name of selected var:
select.name <- selectVar(splsda.srbct, comp = 1)$name
# Then extract the stability values from perf:
stability <- perf.splsda.srbct$features$stable$comp1[select.name]
# Just the head of the stability of the selected var:
head(cbind(selectVar(splsda.srbct, comp = 1)$value, stability))


## ----splsda-indiv12, echo = TRUE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
plotIndiv(splsda.srbct, comp = c(1,2),
          ind.names = FALSE,
          ellipse = TRUE, legend = TRUE,
          star = TRUE,
          title = 'SRBCT, sPLSDA comp 1 - 2')


## ----splsda-indiv23, echo = TRUE, results = 'hide', fig.show = 'hide', fig.path = 'Figures/Part3/'----
plotIndiv(splsda.srbct, comp = c(2,3),
          ind.names = FALSE,
          ellipse = TRUE, legend = TRUE,
          star = TRUE,
          title = 'SRBCT, sPLSDA comp 2 - 3')


## ----splsda-indiv, fig.cap='(ref:splsda-indiv)', echo=FALSE, eval=TRUE, fig.cap='(ref:splsda-indiv)', fig.align='center', out.width = '50%', fig.height=4, fig.ncol = 2, fig.subcap=c('(a)', '(b)')----
knitr::include_graphics(c('Figures/Part3/splsda-indiv12-1.png', 'Figures/Part3/splsda-indiv23-1.png'))


## ----splsda-var, fig.cap='(ref:splsda-var)'----------
var.name.short <- substr(srbct$gene.name[, 2], 1, 10)
plotVar(splsda.srbct, comp = c(1,2), 
        var.names = list(var.name.short), cex = 5)


## ----splsda-plotloading, fig.cap='(ref:splsda-plotloading)'----
plotLoadings(splsda.srbct, comp = 1, method = 'mean', contrib = 'max', 
             name.var = var.name.short)


## ----splsda-cim, fig.width=10, fig.height=8, fig.cap='(ref:splsda-cim)'----
cim(splsda.srbct, row.sideColors = color.mixo(Y))


## ---- results='hold'---------------------------------
set.seed(33) # For reproducibility with this handbook, remove otherwise
train <- sample(1:nrow(X), 50)    # Randomly select 50 samples in training
test <- setdiff(1:nrow(X), train) # Rest is part of the test set

# Store matrices into training and test set:
X.train <- X[train, ]
X.test <- X[test,]
Y.train <- Y[train]
Y.test <- Y[test]

# Check dimensions are OK:
dim(X.train); dim(X.test)


## ----------------------------------------------------
train.splsda.srbct <- splsda(X.train, Y.train, ncomp = 3, keepX = c(20,30,40))


## ----------------------------------------------------
predict.splsda.srbct <- predict(train.splsda.srbct, X.test, 
                                dist = "mahalanobis.dist")


## ----------------------------------------------------
# Just the head:
head(data.frame(predict.splsda.srbct$class, Truth = Y.test))


## ----------------------------------------------------
# Compare prediction on the third component
table(predict.splsda.srbct$class$mahalanobis.dist[,3], Y.test)


## ----------------------------------------------------
# On component 3, just the head:
head(predict.splsda.srbct$predict[, , 3])


## ----splsda-roc, fig.cap='(ref:splsda-roc)', results='hold'----
srbct.auc <- auroc(splsda.srbct)


## ----------------------------------------------------
data(vac18)
X <- vac18$genes
# The treatments
summary(vac18$stimulation)
# The number of repeated measurements per unique individual
summary(as.factor(vac18$sample))


## ----------------------------------------------------
plsda.vac18 <- plsda(X, Y = vac18$stimulation, multilevel = vac18$sample, ncomp = 3)


## ----------------------------------------------------
data('diverse.16S')
# The data are filtered raw counts
X <- diverse.16S$data.raw

# The outcome
Y <- diverse.16S$bodysite
# Unique ID of each individual for multilevel analysis and summary
#summary(as.factor(diverse.16S$sample))
dim(X)


## ----------------------------------------------------
plsda.16S <- plsda(X, Y = diverse.16S$bodysite, 
                  multilevel = diverse.16S$sample, logratio = 'CLR',
                  ncomp = 3)


## ---- eval = TRUE------------------------------------
predict.splsda.srbct2 <- predict(train.splsda.srbct, X.test, dist = "max.dist")


## ---- eval = FALSE-----------------------------------
## data.frame(predict.splsda.srbct2$variates,
##            max.dist = predict.splsda.srbct2$class$max.dist)


## ----splsda-predict, echo=FALSE----------------------
knitr::kable(data.frame(predict.splsda.srbct2$variates, max.dist = predict.splsda.srbct2$class$max.dist), caption = 'Example of predicted components for each dummy variable corresponding to each class, and predicted class based on maximum distance on test samples from the SRBCT data. The true class of the test samples is indicated in the row names.', booktabs = TRUE)


## ---- eval = FALSE-----------------------------------
## predict.splsda.srbct2$centroids


## ----splsda-centroids, echo=FALSE--------------------
knitr::kable(data.frame(predict.splsda.srbct2$centroids),  caption = 'Example of centroids coordinates for each class from the training set, based on the example from the SRBCT data. ', booktabs = TRUE)

