## ----diablo-outline, echo = FALSE, fig.cap ='(ref:diablo-outline)', out.width = '80%'----
knitr::include_graphics("Figures/Part3/DIABLO.pdf")


## ----diablo-design, echo = FALSE, fig.cap ='(ref:diablo-design)', out.width = '80%'----
knitr::include_graphics("Figures/Part3/design_crop_weighted.pdf")


## ----message=FALSE, warning=FALSE--------------------
library(mixOmics)
data(breast.TCGA)
# Extract training data and name each data frame
# Store as list
X <- list(mRNA = breast.TCGA$data.train$mrna, 
          miRNA = breast.TCGA$data.train$mirna, 
          protein = breast.TCGA$data.train$protein)
# Outcome
Y <- breast.TCGA$data.train$subtype
summary(Y)


## ---- eval = FALSE-----------------------------------
## diablo.result1 <- block.plsda(X, Y)         # 1 Run the method
## plotIndiv(diablo.result1)                   # 2 Plot the samples
## plotVar(diablo.result1)                     # 3 Plot the variables


## ---- eval = FALSE-----------------------------------
## # For sparse methods, specify the number of variables to select in each block,
## # for 2 components with names matching block names
## list.keepX <- list(mRNA = c(16, 17), miRNA = c(18,5), protein = c(5, 5))
## list.keepX
## 
## diablo.result <- block.splsda(X, Y, keepX = list.keepX) # 1 Run the method
## plotIndiv(diablo.result)                                # 2 Plot the samples
## plotVar(diablo.result)                                  # 3 Plot the variables


## ----------------------------------------------------
design <- matrix(0.1, ncol = length(X), nrow = length(X), 
                dimnames = list(names(X), names(X)))
diag(design) <- 0
design 


## ---- results='hold'---------------------------------
pls.res1 <- pls(X$mRNA, X$protein, ncomp = 1)
cor(pls.res1$variates$X, pls.res1$variates$Y)

pls.res2 <- pls(X$mRNA, X$miRNA, ncomp = 1)
cor(pls.res2$variates$X, pls.res2$variates$Y)

pls.res3 <- pls(X$protein, X$miRNA, ncomp = 1)
cor(pls.res3$variates$X, pls.res3$variates$Y)


## ----diablo-perf, message=FALSE, fig.cap='(ref:diablo-perf)'----
diablo.tcga <- block.plsda(X, Y, ncomp = 5, design = design)

set.seed(123) # For reproducibility, remove for your analyses
perf.diablo.tcga = perf(diablo.tcga, validation = 'Mfold', folds = 10, nrepeat = 10)

#perf.diablo.tcga$error.rate  # Lists the different error rates

# Plot of the error rates based on weighted vote
plot(perf.diablo.tcga)


## ----------------------------------------------------
perf.diablo.tcga$choice.ncomp$WeightedVote


## ----------------------------------------------------
ncomp <- perf.diablo.tcga$choice.ncomp$WeightedVote["Overall.BER", "centroids.dist"]


## ---- echo = TRUE, eval = FALSE----------------------
## # This code may take several min to run, parallelisation is possible
## set.seed(123) # For reproducibility with this handbook, remove otherwise
## test.keepX <- list(mRNA = c(5:9, seq(10, 25, 5)),
##                    miRNA = c(5:9, seq(10, 20, 2)),
##                    proteomics = c(seq(5, 25, 5)))
## 
## tune.diablo.tcga <- tune.block.splsda(X, Y, ncomp = 2,
##                               test.keepX = test.keepX, design = design,
##                               validation = 'Mfold', folds = 10, nrepeat = 1,
##                               dist = "centroids.dist")


## ---- echo = FALSE, eval = TRUE, warning=FALSE-------
# chunk takes about 2 min to run
set.seed(123) # for reproducibility
test.keepX <- list(mRNA = c(5:9, seq(10, 25, 5)),
                   miRNA = c(5:9, seq(10, 20, 2)),
                   proteomics = c(seq(5, 25, 5)))

tune.diablo.tcga <- tune.block.splsda(X, Y, ncomp = 2, 
                              test.keepX = test.keepX, design = design,
                              validation = 'Mfold', folds = 10, nrepeat = 1, 
                              # use two CPUs for faster computation
                              BPPARAM = BiocParallel::SnowParam(workers = 2),
                              dist = "centroids.dist")


## ----------------------------------------------------
list.keepX <- tune.diablo.tcga$choice.keepX
list.keepX


## ---- message = TRUE---------------------------------
diablo.tcga <- block.splsda(X, Y, ncomp = ncomp, 
                          keepX = list.keepX, design = design)
#diablo.tcga   # Lists the different functions of interest related to that object


## ----------------------------------------------------
diablo.tcga$design


## ---- eval = FALSE-----------------------------------
## # mRNA variables selected on component 1
## selectVar(diablo.tcga, block = 'mRNA', comp = 1)


## ----plot-diablo, message=FALSE, fig.cap='(ref:plot-diablo)'----
plotDiablo(diablo.tcga, ncomp = 1)


## ----diablo-plotindiv, message=FALSE, fig.cap='(ref:diablo-plotindiv)'----
plotIndiv(diablo.tcga, ind.names = FALSE, legend = TRUE, 
          title = 'TCGA, DIABLO comp 1 - 2')


## ----diablo-plotarrow, message=FALSE, fig.cap='(ref:diablo-plotarrow)'----
plotArrow(diablo.tcga, ind.names = FALSE, legend = TRUE, 
          title = 'TCGA, DIABLO comp 1 - 2')


## ----diablo-plotvar, message=FALSE, fig.cap='(ref:diablo-plotvar)'----
plotVar(diablo.tcga, var.names = FALSE, style = 'graphics', legend = TRUE, 
        pch = c(16, 17, 15), cex = c(2,2,2), 
        col = c('darkorchid', 'brown1', 'lightgreen'),
        title = 'TCGA, DIABLO comp 1 - 2')


## ----diablo-circos, message=FALSE, fig.cap='(ref:diablo-circos)'----
circosPlot(diablo.tcga, cutoff = 0.7, line = TRUE, 
           color.cor = c("chocolate3","grey20"), size.labels = 1.5)


## ---- eval = FALSE, message=FALSE--------------------
## # X11()   # Opens a new window
## network(diablo.tcga, blocks = c(1,2,3),
##         cutoff = 0.4,
##         # to save the plot, comment out otherwise
##         save = 'pdf', name.save = 'diablo-network'
##         )


## ----diablo-network, echo = FALSE, fig.cap ='(ref:diablo-network)'----
knitr::include_graphics("Figures/Part3/diablo-network.pdf")


## ----eval = FALSE------------------------------------
## # Not run
## library(igraph)
## myNetwork <- network(diablo.tcga, blocks = c(1,2,3), cutoff = 0.4)
## write.graph(myNetwork$gR, file = "myNetwork.gml", format = "gml")


## ----diablo-loading, message=FALSE, fig.cap='(ref:diablo-loading)'----
plotLoadings(diablo.tcga, comp = 1, contrib = 'max', method = 'median')


## ----diablo-cim, message=FALSE, fig.width=10, fig.height=8, fig.cap='(ref:diablo-cim)'----
cimDiablo(diablo.tcga, color.blocks = c('darkorchid', 'brown1', 'lightgreen'),
          comp = 1, margin=c(8,20), legend.position = "right")


## ----------------------------------------------------
set.seed(123) # For reproducibility with this handbook, remove otherwise
perf.diablo.tcga <- perf(diablo.tcga, 
                   validation = 'Mfold', folds = 10, nrepeat = 10, 
                   dist = 'centroids.dist')

#perf.diablo.tcga  # Lists the different outputs


## ----------------------------------------------------
# Performance with Majority vote
perf.diablo.tcga$MajorityVote.error.rate


## ----------------------------------------------------
# Performance with Weighted vote
perf.diablo.tcga$WeightedVote.error.rate


## ----diablo-auroc, message=FALSE, fig.cap='(ref:diablo-auroc)'----
auc.diablo.tcga <- auroc(diablo.tcga, roc.block = "miRNA", roc.comp = 2,
                   print = FALSE)


## ---- message = FALSE--------------------------------
# Prepare test set data: here one block (proteins) is missing
data.test.tcga <- list(mRNA = breast.TCGA$data.test$mrna, 
                      miRNA = breast.TCGA$data.test$mirna)

predict.diablo.tcga <- predict(diablo.tcga, newdata = data.test.tcga)
# The warning message will inform us that one block is missing
#predict.diablo # List the different outputs


## ----------------------------------------------------
confusion.mat.tcga <- get.confusion_matrix(truth = breast.TCGA$data.test$subtype, 
                     predicted = predict.diablo.tcga$WeightedVote$centroids.dist[,2])
confusion.mat.tcga


## ----------------------------------------------------
get.BER(confusion.mat.tcga)


## ----N-integ, fig.cap='(ref:N-integ)', echo= FALSE, out.width = '80%'----
knitr::include_graphics("Figures/Part3/N-integ-frameworks.pdf")

